@@ -1,5 +1,5 @@
 /* riemann/client.c -- Riemann C client library
- * Copyright (C) 2013, 2014  Gergely Nagy <algernon@madhouse-project.org>
+ * Copyright (C) 2013, 2014, 2015  Gergely Nagy <algernon@madhouse-project.org>
  *
  * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -15,19 +15,24 @@
  * License along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <riemann/client.h>
-
-#include <errno.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 #include <netdb.h>
+#include <stdarg.h>
 
 #include "riemann/_private.h"
 #include "riemann/platform.h"
 
+#include "riemann/client/tcp.h"
+#include "riemann/client/tls.h"
+#include "riemann/client/udp.h"
+
 const char *
 riemann_client_version (void)
 {
@@ -40,24 +45,18 @@ riemann_client_version_string (void)
   return PACKAGE_STRING;
 }
 
-static int _riemann_client_send_message_tcp (riemann_client_t *client,
-                                             riemann_message_t *message);
-static int _riemann_client_send_message_udp (riemann_client_t *client,
-                                             riemann_message_t *message);
-static riemann_message_t *_riemann_client_recv_message_tcp (riemann_client_t *client);
-static riemann_message_t *_riemann_client_recv_message_udp (riemann_client_t *client);
-
 riemann_client_t *
 riemann_client_new (void)
 {
   riemann_client_t *client;
 
-  client = malloc (sizeof (riemann_client_t));
+  client = (riemann_client_t *) malloc (sizeof (riemann_client_t));
 
   client->sock = -1;
   client->srv_addr = NULL;
   client->send = NULL;
   client->recv = NULL;
+  _riemann_client_init_tls (client);
 
   return client;
 }
@@ -68,6 +67,8 @@ riemann_client_disconnect (riemann_client_t *client)
   if (!client || client->sock == -1)
     return -ENOTCONN;
 
+  _riemann_client_disconnect_tls (client);
+
   if (close (client->sock) != 0)
     return -errno;
   client->sock = -1;
@@ -102,13 +103,15 @@ riemann_client_get_fd (riemann_client_t *client)
   return client->sock;
 }
 
-int
-riemann_client_connect (riemann_client_t *client,
-                        riemann_client_type_t type,
-                        const char *hostname, int port)
+static int
+riemann_client_connect_va (riemann_client_t *client,
+                           riemann_client_type_t type,
+                           const char *hostname, int port,
+                           va_list aq)
 {
   struct addrinfo hints, *res;
   int sock;
+  riemann_client_tls_options_t tls_options;
 
   if (!client || !hostname)
     return -EINVAL;
@@ -118,22 +121,31 @@ riemann_client_connect (riemann_client_t *client,
   memset (&hints, 0, sizeof (hints));
   hints.ai_family = AF_UNSPEC;
 
-  if (type == RIEMANN_CLIENT_TCP)
+  switch (type)
     {
-      client->send = _riemann_client_send_message_tcp;
-      client->recv = _riemann_client_recv_message_tcp;
+    case RIEMANN_CLIENT_TCP:
+      _riemann_client_connect_setup_tcp (client, &hints);
+      break;
+    case RIEMANN_CLIENT_UDP:
+      _riemann_client_connect_setup_udp (client, &hints);
+      break;
+    case RIEMANN_CLIENT_TLS:
+      {
+        va_list ap;
+        int e;
 
-      hints.ai_socktype = SOCK_STREAM;
-    }
-  else if (type == RIEMANN_CLIENT_UDP)
-    {
-      client->send = _riemann_client_send_message_udp;
-      client->recv = _riemann_client_recv_message_udp;
+        va_copy (ap, aq);
+        e = _riemann_client_connect_setup_tls (client, &hints, ap, &tls_options);
+        va_end (ap);
 
-      hints.ai_socktype = SOCK_DGRAM;
+        if (e != 0)
+          return e;
+
+        break;
+      }
+    default:
+      return -EINVAL;
     }
-  else
-    return -EINVAL;
 
   if (getaddrinfo (hostname, NULL, &hints, &res) != 0)
     return -EADDRNOTAVAIL;
@@ -164,84 +176,72 @@ riemann_client_connect (riemann_client_t *client,
   client->sock = sock;
   client->srv_addr = res;
 
+  if (type == RIEMANN_CLIENT_TLS)
+    return _riemann_client_connect_tls_handshake (client, &tls_options);
+
   return 0;
 }
 
+int
+riemann_client_connect (riemann_client_t *client,
+                        riemann_client_type_t type,
+                        const char *hostname, int port, ...)
+{
+  va_list ap;
+  int r;
+
+  va_start (ap, port);
+  r = riemann_client_connect_va (client, type, hostname, port, ap);
+  va_end (ap);
+  return r;
+}
+
+int
+riemann_client_connect_1_0 (riemann_client_t *client,
+                            riemann_client_type_t type,
+                            const char *hostname, int port)
+{
+  return riemann_client_connect (client, type, hostname, port);
+}
+
+#ifdef HAVE_VERSIONING
+__asm__(".symver riemann_client_connect_1_0,riemann_client_connect@RIEMANN_C_1.0");
+#endif
+
 riemann_client_t *
 riemann_client_create (riemann_client_type_t type,
-                       const char *hostname, int port)
+                       const char *hostname, int port, ...)
 {
   riemann_client_t *client;
   int e;
+  va_list ap;
 
   client = riemann_client_new ();
 
-  e = riemann_client_connect (client, type, hostname, port);
+  va_start (ap, port);
+  e = riemann_client_connect_va (client, type, hostname, port, ap);
   if (e != 0)
     {
       riemann_client_free (client);
+      va_end (ap);
       errno = -e;
       return NULL;
     }
+  va_end (ap);
 
   return client;
 }
 
-static int
-_riemann_client_send_message_tcp (riemann_client_t *client,
-                                  riemann_message_t *message)
+riemann_client_t *
+riemann_client_create_1_0 (riemann_client_type_t type,
+                           const char *hostname, int port)
 {
-  uint8_t *buffer;
-  size_t len;
-  ssize_t sent;
-
-  buffer = riemann_message_to_buffer (message, &len);
-  if (!buffer)
-    return -errno;
-
-  sent = send (client->sock, buffer, len, 0);
-  if (sent == -1 || (size_t)sent != len)
-    {
-      int e = errno;
-
-      free (buffer);
-      return -e;
-    }
-  free (buffer);
-  return 0;
+  return riemann_client_create (type, hostname, port);
 }
 
-struct _riemann_buff_w_hdr
-{
-  uint32_t header;
-  uint8_t data[0];
-};
-
-static int
-_riemann_client_send_message_udp (riemann_client_t *client,
-                                  riemann_message_t *message)
-{
-  struct _riemann_buff_w_hdr *buffer;
-  size_t len;
-  ssize_t sent;
-
-  buffer = (struct _riemann_buff_w_hdr *)
-    riemann_message_to_buffer (message, &len);
-  if (!buffer)
-    return -errno;
-
-  sent = sendto (client->sock, buffer->data, len - sizeof (buffer->header), 0,
-                 client->srv_addr->ai_addr, client->srv_addr->ai_addrlen);
-  if (sent == -1 || (size_t)sent != len - sizeof (buffer->header))
-    {
-      int e = errno;
-
-      free (buffer);
-      return -e;
-    }
-  free (buffer);
-  return 0;
-}
+#ifdef HAVE_VERSIONING
+__asm__(".symver riemann_client_create_1_0,riemann_client_create@RIEMANN_C_1.0");
+#endif
 
 int
 riemann_client_send_message (riemann_client_t *client,
@@ -270,52 +270,6 @@ riemann_client_send_message_oneshot (riemann_client_t *client,
   return ret;
 }
 
-static riemann_message_t *
-_riemann_client_recv_message_tcp (riemann_client_t *client)
-{
-  uint32_t header, len;
-  uint8_t *buffer;
-  ssize_t received;
-  riemann_message_t *message;
-
-  received = recv (client->sock, &header, sizeof (header), MSG_WAITALL);
-  if (received != sizeof (header))
-    return NULL;
-  len = ntohl (header);
-
-  buffer = malloc (len);
-
-  received = recv (client->sock, buffer, len, MSG_WAITALL);
-  if (received != len)
-    {
-      int e = errno;
-
-      free (buffer);
-      errno = e;
-      return NULL;
-    }
-
-  message = riemann_message_from_buffer (buffer, len);
-  if (message == NULL)
-    {
-      int e = errno;
-
-      free (buffer);
-      errno = e;
-      return NULL;
-    }
-  free (buffer);
-
-  return message;
-}
-
-static riemann_message_t *
-_riemann_client_recv_message_udp (riemann_client_t __attribute__((unused)) *client)
-{
-  errno = ENOTSUP;
-  return NULL;
-}
-
 riemann_message_t *
 riemann_client_recv_message (riemann_client_t *client)
 {
